package dev.micle.xptools.events.common;

import dev.micle.xptools.XpTools;
import dev.micle.xptools.config.Config;
import dev.micle.xptools.operation.OperationCache;
import dev.micle.xptools.operation.OperationItem;
import dev.micle.xptools.operation.OperationUtils;
import net.minecraft.resources.ResourceLocation;
import net.minecraftforge.event.entity.living.LivingExperienceDropEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.registries.ForgeRegistries;

import java.time.Duration;
import java.time.Instant;
import java.util.List;

public class OnLivingExperienceDropEventHandler {
    private final OperationCache cache = OperationCache.entityKillCache;

    @SubscribeEvent
    public void onLivingExperienceDropEvent(LivingExperienceDropEvent event) {
        Instant start = Instant.now();
        float xpToDrop = event.getDroppedExperience();

        // Get Entity id
        ResourceLocation entity_rl = ForgeRegistries.ENTITY_TYPES.getKey(event.getEntity().getType());
        String entity_id = "";
        if (entity_rl != null) {
            entity_id = entity_rl.toString();
        }

        // Get operations
        List<OperationItem> operations = OperationUtils.getOperationList(
                entity_id,
                event.getEntity().getType().getTags().toList(),
                Config.Server.entityKillOperationItems,
                cache
        );

        // Add global operations before all others
        operations.addAll(0, Config.Server.entityKillGlobalOperationItems);

        // Apply operations to xp drops
        xpToDrop = OperationUtils.calculateNewXpAmount(xpToDrop, operations);

        // Debug logging
        if (Config.Server.debugExtra.get()) {
            XpTools.LOGGER.debug("Completed entity kill event:");
            XpTools.LOGGER.debug("\tOperations: {}", operations);
            XpTools.LOGGER.debug("\tCache entries: {}", cache.size());
            XpTools.LOGGER.debug("\tTime taken (nano seconds): {}", Duration.between(start, Instant.now()).toNanos());
            XpTools.LOGGER.debug("\tXP: {} -> {}", event.getDroppedExperience(), xpToDrop);
        }

        // Apply xp drop
        event.setDroppedExperience((int)xpToDrop);
    }
}
